import pprint

from django.core.handlers.wsgi import WSGIRequest
from django.db.models import Manager, QuerySet

from isc_common.http.DSRequest import DSRequest
from django.forms import model_to_dict


class CommonQuerySet(QuerySet):
    pass


class CommonManager(Manager):

    def get_queryset(self):
        return CommonQuerySet(self.model, using=self._db)

    @classmethod
    def getRecord(cls, record):
        return model_to_dict(record)

    def get_range_rows(self, start=None, end=None, function=None):
        if start and end:
            queryResult = super().get_queryset()[start:end]
        elif start:
            queryResult = super().get_queryset()[start:]
        elif end:
            queryResult = super().get_queryset()[:end]
        else:
            queryResult = super().get_queryset()

        if function:
            return [function(record) for record in queryResult]
        else:
            return [model_to_dict(record) for record in queryResult]

    def get_range_rows1(self, request, function=None):
        if isinstance(request, WSGIRequest):
            request = DSRequest(request=request)

        return self.get_range_rows(start=request.startRow, end=request.endRow, function=function)

    def createFromRequest(self, request, printRequest=False):
        if isinstance(request, WSGIRequest):
            request = DSRequest(request=request)
        return super().create(**request.get_data(excluded_keys=['id']))

    def updateFromRequest(self, request, printRequest=False):
        if isinstance(request, WSGIRequest):
            request = DSRequest(request=request)
        data = request.get_data()
        super().filter(id=request.get_id()).update(**data)
        return data

    def deleteFromRequest(self, request, printRequest=False):
        if isinstance(request, WSGIRequest):
            request = DSRequest(request=request)
        return super().filter(id=request.get_id()).delete()

    def get_ex(self, *args, **kwargs):
        """
        Perform the query and return a single object matching the given
        keyword arguments.
        """
        clone = super().get_queryset().filter(*args, **kwargs)
        num = len(clone)
        if num >= 1:
            return clone._result_cache
        if not num:
            return None
        return None
