import logging

from django.contrib.auth.models import User
from django.db.models import Model, BooleanField, DateTimeField, BigAutoField
from django.utils import timezone

from isc_common.fields.related import ForeignKeyProtect
from isc_common.managers.CommonManager import CommonManager

logger = logging.getLogger(__name__)


class NotUnique(Exception):
    pass


class IsBlankStr(Exception):
    pass


class AuditModel(Model):
    id = BigAutoField(primary_key=True, verbose_name="Идентификатор")
    lastmodified = DateTimeField('modified', default=timezone.now)
    deleted = BooleanField(verbose_name="Признак удаления", default=False)
    editing = BooleanField(verbose_name="Возможность редактирования", default=True)
    user_modified = ForeignKeyProtect(User, verbose_name="пользователь выполнивший изменение")

    objects = CommonManager()  # The default manager.

    def save(self, *args, **kwargs):
        super().save(*args, **kwargs)

    class Meta:
        abstract = True
